-- =====================================================
--
--   Library
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.

-- $Id: minila_pkg.vhd,v 1.1 2006/08/25 12:05:06 jsmrz Exp $
--
-- $Log: minila_pkg.vhd,v $
-- Revision 1.1  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


package minila_pkg is

   function wide_and(sigin : std_logic_vector) return std_logic;
   function wide_or(sigin : std_logic_vector) return std_logic;
   function wide_nor(sigin : std_logic_vector) return std_logic;


end package minila_pkg;

package body minila_pkg is

   function wide_and(sigin : std_logic_vector) return std_logic is
      variable and_i	: std_logic;
   begin
      and_i := '1';
      for i in sigin'low to sigin'high loop
         and_i := and_i and sigin(i);
      end loop;
      return(and_i);
   end function wide_and;

   function wide_or(sigin : std_logic_vector) return std_logic is
      variable or_i	: std_logic;
   begin
      or_i := '0';
      for i in sigin'low to sigin'high loop
         or_i := or_i or sigin(i);
      end loop;
      return(or_i);
   end function wide_or;


   function wide_nor(sigin : std_logic_vector) return std_logic is
      variable or_i	: std_logic;
   begin
      or_i := '0';
      for i in sigin'low to sigin'high loop
         or_i := or_i or sigin(i);
      end loop;
      return(not(or_i));
   end function wide_nor;

end package body;

